﻿/*******************************************************************************
*  skST7735x - ST7735チップのTFT-LCD用関数ライブラリ                           *
*              現状はST7735R/Sチップ(緑色タブ)のみ確認した                     *
*                                                                              *
*    begin            - ディスプレイモジュールの初期化を行う処理               *
*    DisplayDirection - ディスプレイの表示方向を指定する処理                   *
*    CLS              - ディスプレイの表示をクリアする処理                     *
*    ColorSet         - 色の設定を行う処理                                     *
*    ReduceColorSet   - RGB値(24ビット色から)の減色を行い色を設定する処理      *
*    PutPixel         - １ドット点を描画する処理                               *
*    Circle           - 指定の円を描画する処理                                 *
*    FillCircle       - 指定の円エリアを指定色で塗りつぶす処理                 *
*    Line             - 指定の線を描画する処理                                 *
*    Box              - 指定の四角形を描画する処理                             *
*    FillBox          - 指定の四角形エリアを指定色で塗りつぶす処理             *
*    Putc             - 指定された位置に１文字(5x8)描画させる処理              *
*    Puts             - 指定された位置に文字列(5x8)を描画させる処理            *
*    Putc2x           - 指定された位置に２倍の１文字(5x8x2)描画させる処理      *
*    Puts2x           - 指定された位置に２倍の文字列(5x8x2)を描画させる処理    *
*    SleepCtl         - スリープのON/OFFを行う処理                             *
*    DisplayINV       - ディスプレイの反転表示を切り替える処理                 *
*                                                                              *
* ============================================================================ *
*   VERSION  DATE        BY             CHANGE/COMMENT                         *
* ---------------------------------------------------------------------------- *
*   1.00     2019-04-22  きむ茶工房     Create                                 *
* ============================================================================ *
*   Arduino IDE 1.8.1 (Ardino Duemilanove 328/UNO)                             *
*******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <SPI.h>
#include "arduino.h"
#include "skST7735x.h"
#include "font.h"        // 5x8のキャラクター文字列データ用ファイル


uint8_t  DSP_Info ;      // チップとディスプレイの情報
uint16_t X_Start ;       // フレームメモリの列開始位置
uint16_t X_End   ;       // フレームメモリの列終了位置
uint16_t Y_Start ;       // フレームメモリの行開始位置
uint16_t Y_End   ;       // フレームメモリの行終了位置
uint8_t  ColorInfo ;     // 色のビット情報
uint8_t  RGB[3] ;        // RGB値からST7735の色情報に変換したテーブル

/*******************************************************************************
*  skST7735x(dspinfo)                                                          *
*    この関数ライブラリを生成する時の初期化処理(コンストラクタ)                *
*    dspinfo : チップとディスプレイの情報                                      *
*******************************************************************************/
skST7735x::skST7735x(uint8_t dspinfo)
{
     DSP_Info = dspinfo ;
}
/*******************************************************************************
*  begin(value)                                                                *
*    ディスプレイモジュールの初期化を行う処理                                  *
*    ここで設定を行っていない機能はデフォルトを使用します。                    *
*    value : 画面の消去色指定  0=黒  1=白                                      *
*******************************************************************************/
void skST7735x::begin(uint8_t value)
{
     // ＳＰＩの初期化
     pinMode(TFT_DC,OUTPUT) ;
     pinMode(TFT_CS,OUTPUT) ;
     SPI.begin() ;                           // ＳＰＩを行う為の初期化
     SPI.setBitOrder(MSBFIRST) ;             // ビットオーダー
     SPI.setClockDivider(SPI_CLOCK_DIV2) ;   // クロック(CLK)をシステムクロックの1/2で使用(16MHz/2)
     SPI.setDataMode(SPI_MODE0) ;            // クロック極性 0(LOW)　クロック位相 1(HIGH)

     // ディスプレイモジュールの初期化(設定していない機能はデフォルトを使用する)
     SendByte(ST7735X_CMD_SWREST,ST7735X_COMMAND) ; // ソフトウェアリセットコマンドの発行
	delay(120) ;
     SendByte(ST7735X_CMD_SLPOUT,ST7735X_COMMAND) ; // スリープから復帰するコマンドの発行
     delay(120) ;

     // 【グリーンタブ】(ディスプレイは横方向に設定)
     if (DSP_Info == (CHIP_ST7735X | DISPLAY_TYPE_1)) {
          // 色の階調を指定するコマンドの発行(18ビットカラーを指定)
          ColorInfo = ST7735X_PARA_18COLOR ;
          SendByte(ST7735X_CMD_COLMOD,ST7735X_COMMAND) ;
          SendByte(ColorInfo,ST7735X_PARAMETER) ;
          // ディスプレイの表示方向を指定する処理
          // フレキシブルケーブルは左側方向(横)
          DisplayDirection(4) ;
     }

     SendByte(ST7735X_CMD_DISPON,ST7735X_COMMAND) ; // ディスプレイを表示するコマンドの発行
     delay(100) ;
     CLS(value) ;                                   // 画面を黒or白で消去する

     RGB[0] = RGB[1] = RGB[2] = 0 ;
}
/*******************************************************************************
*  SendByte(value,dc)                                                          *
*    ディスプレイモジュールに1バイトのデータを送信する処理                     *
*    value : 送信するデータ                                                    *
*    dc    : 送信するデータの種類(0=コマンド 1=パラメータ)                     *
*******************************************************************************/
void skST7735x::SendByte(uint8_t value, uint8_t dc)
{
     digitalWrite(TFT_CS, LOW) ;
     digitalWrite(TFT_DC, dc) ;
     SPI.transfer(value) ;
     digitalWrite(TFT_CS, HIGH) ;
}
/*******************************************************************************
*  SendWord(value)                                                             *
*    ディスプレイモジュールに2バイトのパラメータを送信する処理                 *
*    value : 送信するデータ                                                    *
*******************************************************************************/
void skST7735x::SendWord(uint16_t value)
{
     digitalWrite(TFT_CS, LOW) ;
     digitalWrite(TFT_DC, HIGH) ;
     SPI.transfer((uint8_t)((value >> 8) & 0x00FF)) ;
     SPI.transfer((uint8_t)(value & 0x00FF)) ;
     digitalWrite(TFT_CS, HIGH) ;
}
/*******************************************************************************
*  DisplayInit()                                                               *
*    ディスプレイの範囲(表示画素数)を設定する処理                              *
*    使用するディスプレイに応じて追記しましょう。                              *
*    DisplayDirection()を使うと縦横が入れ替わりますがぁ                        *
*******************************************************************************/
void skST7735x::DisplayInit()
{
     // 【グリーンタブ】(ディスプレイは縦方向に設定)
     if (DSP_Info == (CHIP_ST7735X | DISPLAY_TYPE_1)) {
          // ディスプレイの表示画素数は 128x160(初期状態)
          X_Start = 0 ;
          X_End   = DISPLAY_COLUMN128 - 1 ;
          Y_Start = 0 ;
          Y_End   = DISPLAY_ROW160 - 1 ;
     }
}
/*******************************************************************************
*  DisplayDirection(value)                                                     *
*    ディスプレイの表示方向を指定する処理                                      *
*    value : 表示方向を指定(1-4)                                               *
*******************************************************************************/
void skST7735x::DisplayDirection(uint8_t value)
{
     uint16_t xe , ye ;

     DisplayInit() ;     // ディスプレイの範囲(表示画素数)を設定する

     SendByte(ST7735X_CMD_MADCTL,ST7735X_COMMAND) ;
     switch(value) {
        case 1: // フレキシブルケーブル側が上方向で、左上が0.0(初期状態)
               SendByte(0x00,ST7735X_PARAMETER) ;
               break ;
        case 2: // フレキシブルケーブル側が下方向で、左上が0.0
               SendByte(0xC0,ST7735X_PARAMETER) ;
               break ;
        case 3: // フレキシブルケーブル側右方向で、左上が0.0
               SendByte(0xA0,ST7735X_PARAMETER) ;
               xe = X_End ;
               ye = Y_End ;
               X_End   = ye ;
               Y_End   = xe ;
               break ;
        case 4: // フレキシブルケーブル側左方向で、左上が0.0
               SendByte(0x60,ST7735X_PARAMETER) ;
               xe = X_End ;
               ye = Y_End ;
               X_End   = ye ;
               Y_End   = xe ;
               break ;
     }
}
/*******************************************************************************
*  ans = RangSET(xs,xe,ys,ye)                                                  *
*    書き込むフレームメモリの範囲を指定する処理                                *
*    書き込む場合(表示する場合)は、必ず場所を指定する必要が有ります。          *
*    xs  : 列側開始位置                                                        *
*    xe  : 列側終了位置                                                        *
*    ys  : 行側開始位置                                                        *
*    ye  : 行側終了位置                                                        *
*    ans : 0=正常に設定した  1=指定が範囲外                                    *
*******************************************************************************/
uint8_t skST7735x::RangSET(uint16_t xs, uint16_t xe, uint16_t ys, uint16_t ye)
{
     uint8_t ans ;
     uint16_t x , y ;

     ans = 0 ;
     x = X_End - X_Start ;
     y = Y_End - Y_Start ;
     if ((xs <= x) && (xe <= x) && (ys <= y) && (ye <= y)) {
          // 書き込むフレームメモリの列側範囲を指定する
          SendByte(ST7735X_CMD_CASET,ST7735X_COMMAND) ;
          SendWord(X_Start+xs) ;     // 開始列
          SendWord(X_Start+xe) ;     // 終了列
          // 書き込むフレームメモリの行側範囲を指定する
          SendByte(ST7735X_CMD_RASET,ST7735X_COMMAND) ;
          SendWord(Y_Start+ys) ;     // 開始行
          SendWord(Y_Start+ye) ;     // 終了行
     } else ans = 1 ;
     return ans ;
}
/*******************************************************************************
*  CLS(value)                                                                  *
*    ディスプレイの表示をクリアする処理                                        *
*    この処理は、白か黒でのみ塗りつぶします、色を付けたい場合はFillBoxを使用。 *
*    value : 0=黒  1=白                                                        *
*******************************************************************************/
void skST7735x::CLS(uint8_t value)
{
     uint8_t  c ;
     uint16_t x , i ;

     // 書き込むフレームメモリの範囲を指定
     RangSET(X_Start,X_End,Y_Start,Y_End) ;
     // 範囲内分データをメモリに書き込む
     if (value == 0) c = 0x00 ;
     else {
          if (ColorInfo == ST7735X_PARA_18COLOR) c = 0xFC ;
          else                                   c = 0xFF ;
     }
     x = ((X_End - X_Start) + 1) * ((Y_End - Y_Start) + 1) ;
     SendByte(ST7735X_CMD_RAMWR,ST7735X_COMMAND) ;
     for (i=0 ; i<x ; i++) {
          SendByte(c,ST7735X_PARAMETER) ;
          SendByte(c,ST7735X_PARAMETER) ;
          SendByte(c,ST7735X_PARAMETER) ;
     }
}
/*******************************************************************************
*  ColorSet(red,green,blue)                                                    *
*    色の設定を行う処理                                                        *
*    この処理は、指定されたRGBをST7735用の色構造に変換します、                 *
*    尚、12/16/18ビットカラーのみ対応しています、その他は黒に設定されます。    *
*    12bit(R:4 G:4 B:4)  16bit(R:5 G:6 B:5)  18bit(R:6 G:6 B:6)                *
*    red   : RGBで指定の赤                                                     *
*    green : RGBで指定の緑                                                     *
*    blue  : RGBで指定の青                                                     *
*******************************************************************************/
void skST7735x::ColorSet(uint8_t red, uint8_t green, uint8_t blue)
{
     switch (ColorInfo) {
       case ST7735X_PARA_18COLOR:
            if ((red < 64) && (green < 64) && (blue < 64)) {
                 RGB[0] = red << 2 ;
                 RGB[1] = green << 2 ;
                 RGB[2] = blue << 2 ;
            }
            break ;
       case ST7735X_PARA_16COLOR:
            if ((red < 32) && (green < 64) && (blue < 32)) {
                 RGB[0] = (red << 3) | ((green & 0x38)>>3) ;
                 RGB[1] = ((green & 0x07) << 5) | blue ;
                 RGB[2] = 0 ;
            }
            break ;
       case ST7735X_PARA_12COLOR:
            if ((red < 16) && (green < 16) && (blue < 16)) {
                 RGB[0] = (red << 4) | (green & 0x0F) ;
                 RGB[1] = blue << 4 ;
                 RGB[2] = 0 ;
            }
            break ;
       default:
            RGB[0] = RGB[1] = RGB[2] = 0 ;
            break ;
     }
}
/*******************************************************************************
*  ReduceColorSet(red,green,blue)                                              *
*    RGB値(24ビット色から)の減色を行い色を設定する処理                         *
*    16ビット色は、R(5):G(6):B(5)ビットだがG(5)に統一する                      *
*    red   : RGBで指定(24ビット)の赤                                           *
*    green : RGBで指定(24ビット)の緑                                           *
*    blue  : RGBで指定(24ビット)の青                                           *
*******************************************************************************/
void skST7735x::ReduceColorSet(uint8_t red, uint8_t green, uint8_t blue)
{
     uint8_t c ;

     // 減色色数をチェックする
     if (ColorInfo == ST7735X_PARA_12COLOR) c = ColorInfo + 1 ;
     else c = ColorInfo ;
     c = 8 - c ;
     // 減色し設定する
     ColorSet(red >> c, green >> c, blue >> c) ;
}
/*******************************************************************************
*  PutPixel(xp,yp)                                                             *
*    １ドット点を描画する処理                                                  *
*    xp : ディスプレイ座標の列(横)位置                                         *
*    yp : ディスプレイ座標の行(縦)位置                                         *
*******************************************************************************/
void skST7735x::PutPixel(uint16_t xp, uint16_t yp)
{
     uint8_t ans;

     // 書き込むフレームメモリの範囲を指定
     ans = RangSET(xp,xp,yp,yp) ;
     if (ans == 0) {
          // 指定のフレームメモリ場所に色データを書き込む
          SendByte(ST7735X_CMD_RAMWR,ST7735X_COMMAND) ;
          SendByte(RGB[0],ST7735X_PARAMETER) ;
          SendByte(RGB[1],ST7735X_PARAMETER) ;
          SendByte(RGB[2],ST7735X_PARAMETER) ;
     }
}
/*******************************************************************************
*  Circle(xp,yp,radius)                                                        *
*    指定の円を描画する処理                                                    *
*    xp     : ディスプレイ座標の列(横)位置                                     *
*    yp     : ディスプレイ座標の行(縦)位置                                     *
*    radius : xp/ypを中心として、そこからの半径を指定します                    *
*******************************************************************************/
void skST7735x::Circle(uint16_t xp, uint16_t yp, uint16_t radius)
{
    int16_t f     = 1 - radius ;
    int16_t ddF_x = 1 ;
    int16_t ddF_y = -2 * radius ;
    int16_t x     = 0 ;
    int16_t y     = radius ;

    PutPixel(xp  , yp+radius) ;
    PutPixel(xp  , yp-radius) ;
    PutPixel(xp+radius, yp) ;
    PutPixel(xp-radius, yp) ;

    while (x < y) {
        if (f >= 0) {
            y-- ;
            ddF_y += 2 ;
            f += ddF_y ;
        }
        x++ ;
        ddF_x += 2 ;
        f += ddF_x ;
        PutPixel(xp + x, yp + y) ;
        PutPixel(xp - x, yp + y) ;
        PutPixel(xp + x, yp - y) ;
        PutPixel(xp - x, yp - y) ;
        PutPixel(xp + y, yp + x) ;
        PutPixel(xp - y, yp + x) ;
        PutPixel(xp + y, yp - x) ;
        PutPixel(xp - y, yp - x) ;
    }
}
/*******************************************************************************
*  FillCircle(xp,yp,radius)                                                    *
*    指定の円エリアを指定色で塗りつぶす処理                                    *
*    変更改造すれば楕円も可能です、但し浮動小数点を使わないとダメですがぁ。    *
*    xp     : ディスプレイ座標の列(横)位置                                     *
*    yp     : ディスプレイ座標の行(縦)位置                                     *
*    radius : xp/ypを中心として、そこからの半径を指定します                    *
*******************************************************************************/
void skST7735x::FillCircle(uint16_t xp, uint16_t yp, uint16_t radius)
{
     int16_t iyp1,ixp1,iyp2,ixp2 ;
     int16_t ix,iy,x,y,r,y1 ;

     // 四角形エリアの中心を起点にして半径(radius)の円内を塗りつぶします
     r = radius * radius ;
     ixp1 = xp * 2 ; ixp2 = ixp1 / 2 ;
     iyp1 = yp * 2 ; iyp2 = iyp1 / 2 ;
     for (iy = 0 ; iy < iyp1 ; iy++) {
          y  = iy - iyp2 ;         // "y/係数"で横長楕円になる、係数は0.0-1.0
          y1 = y * y ;
          for(ix = 0 ; ix < ixp1 ; ix++) {
               x = ix - ixp2 ;     // "x/係数"で縦長楕円になる、係数は0.0-1.0
               if ( (x * x) + y1 < r) {
                    // 円の内側なら１ドットを描く
                    PutPixel(ix, iy) ;
               }
          }
     }
}
/*******************************************************************************
*  Line(xsp,ysp,xep,yep)                                                       *
*    指定の線を描画する処理                                                    *
*    xsp/yspを開始点として、xep/yepを終了点とする対角線上に線を描きます。      *
*    xsp : ディスプレイ座標の開始する列(横)位置                                *
*    ysp : ディスプレイ座標の開始する行(縦)位置                                *
*    xep : ディスプレイ座標の終了する列(横)位置                                *
*    yep : ディスプレイ座標の終了する行(縦)位置                                *
*******************************************************************************/
void skST7735x::Line(uint16_t xsp, uint16_t ysp, uint16_t xep, uint16_t yep)
{
     int steep ;
     int deltax, deltay, er ;
     int x, y;
     int ystep;
	
     /// 差分の大きいほうを求める
     steep = (abs(yep - ysp) > abs(xep - xsp)) ;
     /// ｘ、ｙの入れ替え
     if (steep){
          x = xsp; xsp = ysp; ysp = x ;
          x = xep; xep = yep; yep = x ;
     }
     if (xsp > xep) {
          x = xsp; xsp = xep; xep = x ;
          x = ysp; ysp = yep; yep = x ;
     }
     deltax = xep - xsp ;		// 傾き計算
     deltay = abs(yep - ysp) ;
     er = 0 ;
     y  = ysp ;
     /// 傾きでステップの正負を切り替え
     if (ysp < yep) ystep = 1 ; 
     else           ystep = -1 ;
     /// 直線を点で描画
     for (x=xsp ; x <= xep ; x++) {
          if (steep) PutPixel(y,x) ; 
          else       PutPixel(x,y) ;
          er += deltay ;
          if((er << 1) >= deltax) {
               y  += ystep ;
               er -= deltax ;
          }
     }
}
/*******************************************************************************
*  Box(xsp,ysp,xep,yep)                                                        *
*    指定の四角形を描画する処理                                                *
*    xsp/yspを開始の頂点として、xep/yepを終了の頂点とした四角形を描きます。    *
*    xsp : ディスプレイ座標の開始する列(横)位置                                *
*    ysp : ディスプレイ座標の開始する行(縦)位置                                *
*    xep : ディスプレイ座標の終了する列(横)位置                                *
*    yep : ディスプレイ座標の終了する行(縦)位置                                *
*******************************************************************************/
void skST7735x::Box(uint16_t xsp, uint16_t ysp, uint16_t xep, uint16_t yep)
{
     int i ;

     for (i=xsp ; i <= xep ; i++) {
          PutPixel(i,ysp) ;     // 上側横線
          PutPixel(i,yep) ;     // 下側横線
     }
     for (i=ysp+1 ; i <= yep ; i++) {
          PutPixel(xsp,i) ;     // 左側縦線
          PutPixel(xep,i) ;     // 右側縦線
     }
}
/*******************************************************************************
*  FillBox(xsp,ysp,xep,yep)                                                    *
*    指定の四角形エリアを指定色で塗りつぶす処理                                *
*    xsp/yspを開始の頂点として、xep/yepを終了の頂点とした四角形を塗りつぶします*
*    xsp : ディスプレイ座標の開始する列(横)位置                                *
*    ysp : ディスプレイ座標の開始する行(縦)位置                                *
*    xep : ディスプレイ座標の終了する列(横)位置                                *
*    yep : ディスプレイ座標の終了する行(縦)位置                                *
*******************************************************************************/
void skST7735x::FillBox(uint16_t xsp, uint16_t ysp, uint16_t xep, uint16_t yep)
{
     uint16_t i , j ;

     if ((xep-xsp) > (yep-ysp)) {
          // 横方向が長い場合
          for (j=ysp ; j<=yep ; j++) {
               RangSET(xsp,xep,j,j) ;
               SendByte(ST7735X_CMD_RAMWR,ST7735X_COMMAND) ;
               for (i=xsp ; i<=xep ; i++) {
                    SendByte(RGB[0],ST7735X_PARAMETER) ;
                    SendByte(RGB[1],ST7735X_PARAMETER) ;
                    SendByte(RGB[2],ST7735X_PARAMETER) ;
               }
          }
     } else {
          // 縦方向が長い場合
          for (j=xsp ; j<=xep ; j++) {
               RangSET(j,j,ysp,yep) ;
               SendByte(ST7735X_CMD_RAMWR,ST7735X_COMMAND) ;
               for (i=ysp ; i<=yep ; i++) {
                    SendByte(RGB[0],ST7735X_PARAMETER) ;
                    SendByte(RGB[1],ST7735X_PARAMETER) ;
                    SendByte(RGB[2],ST7735X_PARAMETER) ;
               }
          }
     }
}
/*******************************************************************************
*  Putc(xc,yc,c)                                                               *
*    指定された位置に１文字(5x8)描画させる処理                                 *
*    実際の文字太さは5x7ドットで、8ドット目はスペースです。                    *
*    xp : ディスプレイ座標の列(横)位置                                         *
*    yp : ディスプレイ座標の行(縦)位置                                         *
*    c  : 表示する1文字を指定します                                            *
*******************************************************************************/
void skST7735x::Putc(uint16_t xp, uint16_t yp, uint8_t c)
{
     uint8_t  i , j , f ;
     uint16_t x , y  ;

     for (i=0 ; i<5 ; i++) {
          x = i + xp ;
          f = pgm_read_byte(&Font[c-0x20][i]) ;
          for (j=0 ; j<8 ; j++) {
               y = j + yp ;
               if (bitRead(f, j) == 1) PutPixel(x,y) ;
          }
     }
}
/*******************************************************************************
*  Puts(xc,yc,*s)                                                              *
*    指定された位置に文字列(5x8)を描画させる処理                               *
*    xp : ディスプレイ座標の列(横)位置                                         *
*    yp : ディスプレイ座標の行(縦)位置                                         *
*    *s : 表示する文字列を指定します(文字列の最後はnullでなければなりません)   *
*******************************************************************************/
void skST7735x::Puts(uint16_t xp, uint16_t yp, uint8_t *s)
{
     while(*s) {
          Putc(xp,yp,*s++) ;
          xp = xp + 6 ;       // 横を1ドット空ける為に5+1=6とする
     }
}
/*******************************************************************************
*  Putc2x(xc,yc,c)                                                             *
*    指定された位置に２倍の１文字(5x8x2)描画させる処理                         *
*    xp : ディスプレイ座標の列(横)位置                                         *
*    yp : ディスプレイ座標の行(縦)位置                                         *
*    c  : 表示する1文字を指定します                                            *
*******************************************************************************/
void skST7735x::Putc2x(uint16_t xp, uint16_t yp, uint8_t c)
{
     uint8_t  i , j , f ;
     uint16_t x , y , x1 , y1 ;

     for (i=0 ; i<5 ; i++) {
          x = ((i*2) + xp) ;
          x1 = x + 1 ;
          f = pgm_read_byte(&Font[c-0x20][i]) ;
          for (j=0 ; j<8; j++) {
               y = ((j*2) + yp) ;
               y1 = y + 1 ;
               if (bitRead(f, j) == 1) {
                    PutPixel(x,y) ;
                    PutPixel(x1,y) ;
                    PutPixel(x,y1) ;
                    PutPixel(x1,y1) ;
               }
          }
     }
}
/*******************************************************************************
*  Puts2x(xc,yc,*s)                                                            *
*    指定された位置に２倍の文字列(5x8x2)を描画させる処理                       *
*    xp : ディスプレイ座標の列(横)位置                                         *
*    yp : ディスプレイ座標の行(縦)位置                                         *
*    *s : 表示する文字列を指定します(文字列の最後はnullでなければなりません)   *
*******************************************************************************/
void skST7735x::Puts2x(uint16_t xp, uint16_t yp, uint8_t *s)
{
     while(*s) {
          Putc2x(xp++,yp,*s++) ;
          xp = xp + 11 ;      // 横を1ドット空ける為に5+5+1=11とする
     }
}
/*******************************************************************************
*  SleepCtl(sel)                                                               *
*    スリープのON/OFFを行う処理                                                *
*    復帰すればその前までの画面が表示されます、スリープ時に画面表示をOFFだがぁ *
*    バックライトは消えない、やり方がわからない。                              *
*    sel : 0= スリープからの復帰  1= スリープモードにする                      *
*******************************************************************************/
void skST7735x::SleepCtl(uint8_t sel)
{
     if (sel == 0) {
          SendByte(ST7735X_CMD_SLPOUT,ST7735X_COMMAND) ;    // スリープから復帰する
          delay(120) ;
          SendByte(ST7735X_CMD_DISPON,ST7735X_COMMAND) ;    // 画面表示ＯＮ 
          delay(100) ;
     } else {
          SendByte(ST7735X_CMD_DISPOFF,ST7735X_COMMAND) ;   // 画面表示ＯＦＦ
          SendByte(ST7735X_CMD_SLPIN,ST7735X_COMMAND) ;     // スリープを行う
          delay(120) ;
     }
}
/*******************************************************************************
*  DisplayINV(sel)                                                             *
*    ディスプレイの反転表示を切り替える処理                                    *
*    この処理を行った時点でフレームメモリ全体(画面全体)ビットが反転して表示    *
*    されます、即ち色が白だった場合は黒で表示します。                          *
*    sel : 0= 反転表示ＯＦＦ  1= 反転表示ＯＮ                                  *
*******************************************************************************/
void skST7735x::DisplayINV(uint8_t sel)
{
     if (sel == 0) {
          SendByte(ST7735X_CMD_INVOFF,ST7735X_COMMAND) ; // 反転表示ＯＦＦするコマンドの発行
     } else {
          SendByte(ST7735X_CMD_INVON,ST7735X_COMMAND) ;  // 反転表示ＯＮするコマンドの発行
     }
}
